package	D2P::Generator::PHPClasses;
use base 'D2P::Generator';
use D2P::Parser;

sub	strToPgId
{
	my	$string	= shift;

	return $string if $string =~ /^\".*\"$/;

	return "\"$string\"" if (length $string < 32);

	$string =~ s/\_+/_/g;
	return "\"$string\"" if (length $string < 32);

	my	@list	= split /_/, $string;
	my	$old	= $string;
	my	$lsz	= @list;
	my	$rest	= 32 - @list;
	my	$max;

	$string = "";
	foreach my $it (@list)
	{
		$max = $rest / $lsz;
		$max =~ s/\.(.*)$//;
		$max += 1 if ($1 && $1 > 5 && $max < $rest);
		$string .= '_' if $string;

		if	(length $it > $max)
		{
			$string .= substr $it, 0, $max;
			$rest -= $max;
		}
		else
		{
			$string .= $it;
			$rest -= length $string;
		}

		$lsz--;
	}

	print SQLOUT STDERR "WARNING: name \"$old\" changed to \"$string\"\n";
	return "\"$string\"";
}


###
## Strip double quotes from a Postgres name
#
sub	stripDQuotes
{
	my $n = shift;
	$n =~ s/\"//g;
	return	$n;
}


sub	init
{
	my	$this	= shift;

	$this->{config}->{phpout} = "";
}


my $phpout = "";

sub	commandLine
{
	return	(
		"-p|--php"	=> {
			desc	=> "Output PHP mirror classes to the specified file.",
			uniq	=> 1,
			args	=> ["phpout"]
		}
	);
}


###
## Create complete PHP output
#
sub	generate
{
	my	$this = shift;
	$phpout = $this->{config}->{phpout};

	return unless $phpout;
	unless	(open(PHPOUT, ">$phpout"))
	{
		print STDERR "Warning : could not create PHP output file $phpout\n";
		return;
	}
	&createPHPHeader();
	&createPHPEnums();
	&createPHPClasses();
	&createPHPFooter();
	close PHPOUT;
}


###
## Create PHP header
#
sub	createPHPHeader
{
	$DEBUG && print STDERR " Writing header\n";

	print PHPOUT "<?php\n\n/*\n";

	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'box' && $g->{topLevelText} ne '' && @{$g->{classes}} == 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\n$//;

		my	@lines = split /\n/, $title;
		foreach my $l (@lines)
			{ print PHPOUT "$l\n"; }
		print PHPOUT "\n";
		last;
	}

	print PHPOUT "Input file  : $infile\n";
	print PHPOUT "\n";
	print PHPOUT "Defines PHP classes for the following modules : \n";
	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'box' && $g->{topLevelText} ne '' && @{$g->{classes}} != 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\s+/ /g;

		print PHPOUT " - $title\n";
	}

	my	@requiredModules = ();
	foreach my $g (@groups)
	{
		next unless ($g->{type} eq 'pack' && $g->{topLevelText} ne '' && @{$g->{classes}} != 0);

		my	$title = $g->{topLevelText};
		$title =~ s/\s+/ /g;
		push @requiredModules, $title;
	}
	if	(@requiredModules)
	{
		print PHPOUT "\n";
		print PHPOUT "Requires the following modules :\n - ";
		print PHPOUT join ("\n - ", @requiredModules);
		print PHPOUT "\n";
	}
	print PHPOUT "*/\n\nrequire_once(\"d2p_manager.inc\");\n";
	print PHPOUT "require_once(\"d2p_data.inc\");";
	print PHPOUT "\n\n";
}


###
## Create PHP classes for enumerations according to data created by checkAndModifyInput()
#
sub	createPHPEnums
{
	foreach my $re (@enumerations)
	{
		my	$l = 0;
		my	$enum = {%$re};
		$enum->{tablename} =~ s/\"//g;
		foreach my $w1 (@{$enum->{contents}})
		{
			my	$w1l = length($w1);
			($w1l > $l) && ($l = $w1l);
		}

		print PHPOUT <<"ENDOFENUM";

// Enumeration class for member $enum->{fieldname}

class	d2p_cl_$enum->{tablename} extends d2p_enum_data
{
	function	d2p_cl_$enum->{tablename}(&\$manager, \$identifier = NULL)
	{
		\$this->d2p_data(\$manager, "$enum->{tablename}", "id");

		if      (!is_null(\$identifier))
		{
			\$db = \$this->manager->getDatabaseConnection();
			\$query = "SELECT * FROM \\"$enum->{tablename}\\" WHERE \\"id\\" = ";
			\$query .= \$this->fieldToSQL("int", \$identifier);
			\$result = \@pg_query(\$db, \$query);
			if      (!\$result)
			{
				\$this->errorMsg = "Unable to execute query : \$query";
				return  NULL;
			}
			\$row = pg_fetch_array(\$result);
			\$this->loadDatabaseEntry(\$row);
		}
	}
}
ENDOFENUM
	}
}


###
## Create PHP classes for each table
#
sub	createPHPClasses
{
	foreach my $gro (sort keys %groupsOrdered)
	{
		foreach my $g (@{$groupsOrdered{$gro}})
		{
			next unless ($g->{type} eq 'box' && @{$g->{classes}});

			my	$title = $g->{topLevelText};
			$title =~ s/\n/\n\/\/ /g;
			print PHPOUT "\n\n\n// Classes for module $title\n\n";
			foreach my $o (@{$g->{classes}})
			{
				my	$x = 0;
				my	@pkey = ();
				my	@codeMethodes = ();
				my	@codeChamps = ();
				my	@codeFormulaire = ();
				my	$tnam = &stripDQuotes(&strToPgId($o->{name}));

				print PHPOUT "class d2p_cl_$tnam extends ";
				if	(@{$o->{inherits}})
				{
					print PHPOUT "d2p_cl_" . &stripDQuotes(&strToPgId($o->{inherits}->[0]->{name}));
				}
				else
				{
					print PHPOUT "d2p_data";
				}
				print PHPOUT "\n{\n\tfunction\td2p_cl_$tnam(\&\$manager, \$identifier = NULL)\n\t{\n\t\t";
				print PHPOUT '$this->d2p_data($manager, ';
				{
					my	@opk = split /,/, $o->{pkey};
					@opk = map { &stripDQuotes(&strToPgId($_)) } (@opk);
					print PHPOUT '"' . $tnam . '", "' . $opk[0] . '");';
				}
				print PHPOUT <<"EOF";


		if      (!is_null(\$identifier))
		{
			\$db = \$this->manager->getDatabaseConnection();
			\$query = "SELECT * FROM \\"" . \$this->table . "\\" WHERE \\"" . \$this->identifierName . "\\" = ";
			\$query .= \$this->fieldToSQL(
				\$this->fields[\$this->identifierName]["type"], \$identifier
			);
			\$result = \@pg_query(\$db, \$query);
			if      (!\$result)
			{
				\$this->erreurFormat = "Unable to execute query : \$query";
				return  NULL;
			}
			\$row = pg_fetch_array(\$result);
			\$this->loadDatabaseEntry(\$row);
		}
	}

	function	prepareFields()
	{
		parent::prepareFields();
EOF

				foreach my $f (@{$o->{fields}})
				{
					my	$fnam = &stripDQuotes(&strToPgId($f->{name}));
					print PHPOUT "\t\t" . '$this->fields["' . $fnam . '"] = array(';
					print PHPOUT "\n\t\t\t\"initial_value\"\t=> NULL,";
					print PHPOUT "\n\t\t\t\"current_value\"\t=> NULL,";
					print PHPOUT "\n\t\t\t\"type\"\t\t=> \"";
					print PHPOUT ($f->{type} ne "serial" ? $f->{type} : "int");
					print PHPOUT "\",";

					if	($f->{references})
					{
						my	($table, $field) = map { &stripDQuotes(&strToPgId($_)); } (split (/\./, $f->{references}));
						print PHPOUT "\n\t\t\t\"references\"\t=> \"$table\",";
					}

					print PHPOUT "\n\t\t\t\"mandatory\"\t=> " . ($f->{not_null} ? "true" : "false") . "\n\t\t);\n";
					push @codeMethodes, "\n";
					push @codeMethodes, "\tfunction\tg_$fnam()\n";
					push @codeMethodes, "\t{\n";
					push @codeMethodes, "\t\treturn\t\$this->getField(\"$fnam\");\n";
					push @codeMethodes, "\t}\n";
					push @codeMethodes, "\n";
					push @codeMethodes, "\tfunction\ts_$fnam(\$value)\n";
					push @codeMethodes, "\t{\n";
					### FIXME : add type checks etc...
					push @codeMethodes, "\t\t\$this->setField(\"$fnam\", \$valeur);\n";
					push @codeMethodes, "\t}\n";

					push @codeFormulaire, "\t\t\$form .= \"<TR><TD>\\n\";\n";
					if	($f->{not_null})
					{
						push @codeFormulaire, "\t\t\$form .= \"<FONT COLOR='red'>\\n\";\n";
					}
					my $nomPresentable = $f->{name};
					$nomPresentable =~ s/_/ /g;
					push @codeFormulaire, "\t\t\$form .= \"$nomPresentable\\n\";\n";
					if	($f->{not_null})
					{
						push @codeFormulaire, "\t\t\$form .= \"</FONT>\\n\";\n";
					}
					push @codeFormulaire, "\t\t\$form .= \"</TD><TD>\\n\";\n";
					
					if	($f->{references})
					{
						my	($table, $field) = map { &stripDQuotes(&strToPgId($_)); } (split (/\./, $f->{references}));
						push @codeFormulaire, "\t\t\$form .= \"<SELECT NAME='form__$tnam\__$fnam'>\\n\";\n";
						push @codeFormulaire, "\t\t\$tab = \$this->manager->listFromBase"
							. "(\"$table\", \"SELECT \\\"$field\\\" FROM \\\"$table\\\"\");\n";
						push @codeFormulaire, "\t\tif\t(is_null(\$tab))\n";
						push @codeFormulaire, "\t\t\t\$form .= \"<OPTION VALUE=''>No records</OPTION>\\n\";\n";
						push @codeFormulaire, "\t\telse\n";
						push @codeFormulaire, "\t\t{\n";
						push @codeFormulaire, "\t\t\t\$form .= \"<OPTION VALUE=''>No record selected</OPTION>\\n\";\n";
						push @codeFormulaire, "\t\t\t\$o = \$this->getField(\"$fnam\");\n";
						push @codeFormulaire, "\t\t\tfor\t(\$i=0;\$i<count(\$tab);\$i++)\n";
						push @codeFormulaire, "\t\t\t{\n";
						push @codeFormulaire, "\t\t\t\t\$form .= \"<OPTION VALUE='\" . "
							. "\$tab[\$i]->getIdentifier() . \"'\";\n";
						push @codeFormulaire, "\t\t\t\tif\t(\$tab[\$i]->getObjId() == \$o->getObjId())\n";
						push @codeFormulaire, "\t\t\t\t\t\$form .= \" SELECTED\";\n";
						push @codeFormulaire, "\t\t\t\t\$form .= \">\" . "
							. "\$tab[\$i]->getDescription() . \"</OPTION>\\n\";\n";
						push @codeFormulaire, "\t\t\t}\n";
						push @codeFormulaire, "\t\t}\n";
						push @codeFormulaire, "\t\t\$form .= \"</SELECT>\\n\";\n";
					}
					else
					{
						push @codeFormulaire, "\t\t\$form .= \"<INPUT TYPE='text' NAME='form__$tnam";
						push @codeFormulaire, "__" . $fnam . "' VALUE=\\\"\";\n";
						push @codeFormulaire, "\t\t\$form .= \$this->getField(\"$fnam\");\n";
						push @codeFormulaire, "\t\t\$form .= \"\\\">\\n\";\n";
					}
					push @codeFormulaire, "\t\t\$form .= \"</TD></TR>\\n\";\n";
				}
				print PHPOUT "\t}\n";
				print PHPOUT join("", @codeMethodes);
				print PHPOUT "\n\tfunction\tgetHTMLForm()\n";
				print PHPOUT "\t{\n";
				print PHPOUT "\t\t\$form = parent::getHTMLForm();\n";
				print PHPOUT join("", @codeFormulaire);
				print PHPOUT "\t\treturn\t\$form;\n";
				print PHPOUT "\t}\n";
				print PHPOUT "}\n\n\n";
			}
		}
	}
}


###
## Create PHP footer
#
sub	createPHPFooter
{
	$DEBUG && print STDERR " Writing footer\n";
	print PHPOUT "\n\n// PHP code generated by dia2postgres.pl\n";
	print PHPOUT "// $VERSION\n"; 
	print PHPOUT "// on " . `date`;
	print PHPOUT "\n?>\n";
}




1;
